      REAL FUNCTION SPHDIS ( N, X )
C
C Distance between two arbitrary points on the surfaces of
C two spheres.
C
C This is an auxiliary subroutine to test the minimization
C subroutine FMINSI
C
C   FMINSI - Fortran subroutines for unconstrained function minimization
C   Copyright (C) 1989, 2001  Hugo Pfoertner
C
C   This library is free software; you can redistribute it and/or
C   modify it under the terms of the GNU Lesser General Public
C   License as published by the Free Software Foundation; either
C   version 2.1 of the License, or (at your option) any later version.
C
C   This library is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
C   Lesser General Public License for more details.
C
C   You should have received a copy of the GNU Lesser General Public
C   License along with this library; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
C   USA
C
C   Contact info: mailto:hugo@pfoertner.org
C   or use the information provided at http://www.pfoertner.org/
C
C Author: Hugo Pfoertner, Oberhaching, Germany
C
C Description:
C The first sphere is located at the origin and has a radius 1.
C The second sphere has center (0,0,OFFSET) and a radius RHO
C OFFSET and RHO are passed to the function via common block /SPHPAR/
C The coordinates of the points (Point 1 on sphere 1, point 2 on sphere 
C 2) are passed as spherical coordinates via the vector X
C (X1<->PHI1, X2<->THETA1, X3<->PHI2, X4<->THETA2)
C The function returns the cartesian distance between P1 and P2.
C A penalty term is added, if any of the 4 angles leaves the range +-PI
C
C Version History:
C 03.06.01 English translation of comments, LGPL header added
C 15.06.89 Initial version
C
      INTEGER N
      REAL X(N)
      COMMON /SPHPAR/ OFFSET, RHO
      COMMON /ACTPRN/ IPRN, IFCCNT
C
      PARAMETER (PI=3.14159265358, DP=180.0/PI)
C
C Increase count
      IFCCNT = IFCCNT + 1
C
C A penalty term is applied to the distance function if any angle
C passed via X is outside the range +-PI
C
      PENPI = 0.0
C
      DO 10 I = 1, N
      IF ( ABS (X(I)) .GT. PI ) THEN
        PENPI = PENPI + ( ABS(X(I)) - PI ) ** 2
      ENDIF
10    CONTINUE
C
      PENPI = 2.0 * PENPI * MAX ( 1.0, OFFSET, RHO )
C
C Convert spherical coordinates into cartesian coordinates
C First sphere with radius 1
C
      X1 = SIN (X(2)) * COS (X(1))
      Y1 = SIN (X(2)) * SIN (X(1))
      Z1 = COS (X(2))
C
C Second sphere with radius RHO and Z-offset OFFSET
C
      X2 = RHO * SIN (X(4)) * COS (X(3))
      Y2 = RHO * SIN (X(4)) * SIN (X(3))
      Z2 = RHO * COS (X(4)) + OFFSET
C
C Return value = distance plus penalty for angles exceeding +-PI
C
      SPHDIS = PENPI + SQRT ( (X2-X1)**2 + (Y2-Y1)**2 + (Z2-Z1)**2 )
C
C Optional printout
C
      IF ( IPRN .LT. 0 ) THEN
        WRITE (*,1000) IFCCNT, SPHDIS,
     &                 X(1)*DP, X(2)*DP, X(3)*DP, X(4)*DP
1000  FORMAT ( ' NF:', I5, ' F:', 1PE13.6, ' X:', 4(0PF11.5) )
        IF ( PENPI .GT. 0.0 )
     &    WRITE (*,*) ' Penalty on function:', PENPI
      ENDIF
C
      RETURN
C End of function SPHDIS
      END
